# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2003-2006 Gary Bishop.
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
# table for translating virtual keys to X windows key symbols
from __future__ import print_function, unicode_literals, absolute_import

try:
    set
except NameError:
    from sets import Set as set
    
from pyreadline.unicode_helper import ensure_unicode

validkey =set(['cancel',      'backspace',    'tab',          'clear',
               'return',      'shift_l',      'control_l',    'alt_l',
               'pause',       'caps_lock',    'escape',       'space',
               'prior',       'next',         'end',          'home',
               'left',        'up',           'right',        'down',
               'select',      'print',        'execute',      'snapshot',
               'insert',      'delete',       'help',         'f1',
               'f2',          'f3',           'f4',           'f5',
               'f6',          'f7',           'f8',           'f9',
               'f10',         'f11',          'f12',          'f13',
               'f14',         'f15',          'f16',          'f17',
               'f18',         'f19',          'f20',          'f21',
               'f22',         'f23',          'f24',          'num_lock',
               'scroll_lock', 'vk_apps',      'vk_processkey','vk_attn',
               'vk_crsel',    'vk_exsel',     'vk_ereof',     'vk_play',
               'vk_zoom',     'vk_noname',    'vk_pa1',       'vk_oem_clear',
               'numpad0',     'numpad1',      'numpad2',      'numpad3',
               'numpad4',     'numpad5',      'numpad6',      'numpad7',
               'numpad8',     'numpad9',      'divide',       'multiply',
               'add',         'subtract',     'vk_decimal'])

escape_sequence_to_special_key = {"\\e[a" : "up", "\\e[b" : "down", "del" : "delete"}

class KeyPress(object):
    def __init__(self, char="", shift=False, control=False, meta=False, keyname=""):
        if control or meta or shift:
            char = char.upper()
        self.info = dict(char=char,
                         shift=shift,
                         control=control,
                         meta=meta,
                         keyname=keyname)
        
    def create(name):
        def get(self):
            return self.info[name]

        def set(self, value):
            self.info[name] = value
        return property(get, set)
    char = create("char")
    shift = create("shift")
    control = create("control")
    meta = create("meta")
    keyname = create("keyname")
        
    def __repr__(self):
        return "(%s,%s,%s,%s)"%tuple(map(ensure_unicode, self.tuple()))

    def tuple(self):
        if self.keyname:
            return (self.control, self.meta, self.shift, self.keyname)
        else:
            if self.control or self.meta or self.shift:
                return (self.control, self.meta, self.shift, self.char.upper())
            else:
                return (self.control, self.meta, self.shift, self.char)

    def __eq__(self, other):
        if isinstance(other, KeyPress):
            s = self.tuple()
            o = other.tuple()
            return s == o
        else:
            return False

def make_KeyPress_from_keydescr(keydescr):
    keyinfo = KeyPress()
    if len(keydescr) > 2 and keydescr[:1] == '"' and keydescr[-1:] == '"':
        keydescr = keydescr[1:-1]
        
    while 1:
        lkeyname = keydescr.lower()
        if lkeyname.startswith('control-'):
            keyinfo.control = True
            keydescr = keydescr[8:]
        elif lkeyname.startswith('ctrl-'):
            keyinfo.control = True
            keydescr = keydescr[5:]
        elif keydescr.lower().startswith('\\c-'):
            keyinfo.control = True
            keydescr = keydescr[3:]
        elif keydescr.lower().startswith('\\m-'):
            keyinfo.meta = True
            keydescr = keydescr[3:]
        elif keydescr in escape_sequence_to_special_key:
            keydescr = escape_sequence_to_special_key[keydescr]
        elif lkeyname.startswith('meta-'):
            keyinfo.meta = True
            keydescr = keydescr[5:]
        elif lkeyname.startswith('alt-'):
            keyinfo.meta = True
            keydescr = keydescr[4:]
        elif lkeyname.startswith('shift-'):
            keyinfo.shift = True
            keydescr = keydescr[6:]
        else:
            if len(keydescr) > 1:
                if keydescr.strip().lower() in validkey:
                    keyinfo.keyname = keydescr.strip().lower()
                    keyinfo.char = ""
                else:
                    raise IndexError("Not a valid key: '%s'"%keydescr)
            else:
                keyinfo.char = keydescr
            return keyinfo

if __name__ == "__main__":
    import startup
    


# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2007  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import sys

from .py3k_compat import unicode, bytes

try:
    pyreadline_codepage = sys.stdout.encoding
except AttributeError:        
    # This error occurs when pdb imports readline and doctest has replaced 
    # stdout with stdout collector. We will assume ascii codepage
    pyreadline_codepage = "ascii"

if pyreadline_codepage is None:  
    pyreadline_codepage = "ascii"

if sys.version_info < (2, 6):
    bytes = str

PY3 = (sys.version_info >= (3, 0))

def ensure_unicode(text):
    """helper to ensure that text passed to WriteConsoleW is unicode"""
    if isinstance(text, bytes):
        try:
            return text.decode(pyreadline_codepage, "replace")
        except (LookupError, TypeError):
            return text.decode("ascii", "replace")
    return text


def ensure_str(text):
    """Convert unicode to str using pyreadline_codepage"""
    if isinstance(text, unicode):
        try:
            return text.encode(pyreadline_codepage, "replace")
        except (LookupError, TypeError):
            return text.encode("ascii", "replace")
    return text

def biter(text):
    if PY3 and isinstance(text, bytes):
        return (s.to_bytes(1, 'big') for s in text)
    else:
        return iter(text)



# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import

import re, operator


def str_find_all(str, ch):
    result = []
    index = 0
    while index >= 0:
        index = str.find(ch, index)
        if index >= 0:
            result.append(index)
            index += 1
    return result
        

word_pattern = re.compile("(x*)")

def markwords(str, iswordfun):
    markers = {True : "x", False : "o"}
    return "".join([markers[iswordfun(ch)] for ch in str])

def split_words(str, iswordfun):
    return [x for x in word_pattern.split(markwords(str,iswordfun)) if x != ""]

def mark_start_segment(str, is_segment):
    def mark_start(s):
        if s[0:1] == "x":
            return "s" + s[1:]
        else:
            return s
    return "".join(map(mark_start, split_words(str, is_segment)))

def mark_end_segment(str, is_segment):
    def mark_start(s):
        if s[0:1] == "x":
            return s[:-1] + "s"
        else:
            return s
    return "".join(map(mark_start, split_words(str, is_segment)))
    
def mark_start_segment_index(str, is_segment):
    return str_find_all(mark_start_segment(str, is_segment), "s")

def mark_end_segment_index(str, is_segment):
    return [x + 1 for x in str_find_all(mark_end_segment(str, is_segment), "s")]


################  Following are used in lineobj  ###########################

def is_word_token(str):
    return not is_non_word_token(str)
    
def is_non_word_token(str):
    if len(str) != 1 or str in " \t\n":
        return True
    else:
        return False

def next_start_segment(str, is_segment):
    str = "".join(str)
    result = []
    for start in mark_start_segment_index(str, is_segment):
        result[len(result):start] = [start for x in range(start - len(result))]
    result[len(result):len(str)] = [len(str) for x in range(len(str) - len(result) + 1)]            
    return result
    
def next_end_segment(str, is_segment):
    str = "".join(str)
    result = []
    for start in mark_end_segment_index(str, is_segment):
        result[len(result):start] = [start for x in range(start - len(result))]
    result[len(result):len(str)] = [len(str) for x in range(len(str) - len(result) + 1)]            
    return result    


def prev_start_segment(str, is_segment):
    str = "".join(str)
    result = []
    prev = 0
    for start in mark_start_segment_index(str, is_segment):
        result[len(result):start+1] = [prev for x in range(start - len(result) + 1)]
        prev=start
    result[len(result):len(str)] = [prev for x in range(len(str) - len(result) + 1)]            
    return result

def prev_end_segment(str, is_segment):
    str = "".join(str)
    result = []
    prev = 0
    for start in mark_end_segment_index(str, is_segment):
        result[len(result):start + 1] = [prev for x in range(start - len(result) + 1)]
        prev=start
    result[len(result):len(str)] = [len(str) for x in range(len(str) - len(result) + 1)]            
    return result    




# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import

import socket, logging, logging.handlers
from pyreadline.unicode_helper import ensure_str

host = "localhost"
port = logging.handlers.DEFAULT_TCP_LOGGING_PORT


pyreadline_logger = logging.getLogger('PYREADLINE')
pyreadline_logger.setLevel(logging.DEBUG)
pyreadline_logger.propagate = False
formatter = logging.Formatter(str('%(message)s'))
file_handler = None

class NULLHandler(logging.Handler):
    def emit(self, s):
        pass

class SocketStream(object):
    def __init__(self, host, port):
        self.logsocket = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)

    def write(self, s):
        self.logsocket.sendto(ensure_str(s), (host, port))

    def flush(self):
        pass

    def close(self):
        pass

socket_handler = None
pyreadline_logger.addHandler(NULLHandler())

def start_socket_log():
    global socket_handler
    socket_handler = logging.StreamHandler(SocketStream(host, port))
    socket_handler.setFormatter(formatter)
    pyreadline_logger.addHandler(socket_handler)

def stop_socket_log():
    global socket_handler
    if socket_handler:
        pyreadline_logger.removeHandler(socket_handler)
        socket_handler = None

def start_file_log(filename):
    global file_handler
    file_handler = logging.FileHandler(filename, "w")
    pyreadline_logger.addHandler(file_handler)

def stop_file_log():
    global file_handler
    if file_handler:
        pyreadline_logger.removeHandler(file_handler)
        file_handler.close()
        file_handler = None

def stop_logging():
    log("STOPING LOG")
    stop_file_log()
    stop_socket_log()

def log(s):
    s = ensure_str(s)
    pyreadline_logger.debug(s)



print("import: 'pyreadline'")
import pyreadline

print("import: 'pyreadline.clipboard'")
import pyreadline.clipboard

print("import: 'pyreadline.console'")
import pyreadline.console

print("import: 'pyreadline.keysyms'")
import pyreadline.keysyms

print("import: 'pyreadline.lineeditor'")
import pyreadline.lineeditor

print("import: 'pyreadline.modes'")
import pyreadline.modes




# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Michael Graz. <mgraz@plan10.com>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import
from pyreadline.modes.emacs import *
from pyreadline import keysyms
from pyreadline.lineeditor import lineobj
from pyreadline.keysyms.common import make_KeyPress_from_keydescr

import unittest
class MockReadline:
    def __init__ (self):
        self.l_buffer=lineobj.ReadLineTextBuffer("")
        self._history=history.LineHistory()

    def add_history (self, line):
        self._history.add_history (lineobj.TextLine (line))

    def _print_prompt (self):
        pass

    def _bell (self):
        pass

    def insert_text(self, string):
        '''Insert text into the command line.'''
        self.l_buffer.insert_text(string)


class MockConsole:
    def __init__ (self):
        self.bell_count = 0
        self.text = ''

    def size (self):
        return (1, 1)

    def cursor(self, visible=None, size=None):
        pass

    def bell (self):
        self.bell_count += 1

    def write (self, text):
        self.text += text




class Event:
    def __init__ (self, char):
        if char=="escape":
            self.char='\x1b'
        elif char=="backspace":
            self.char='\x08'
        elif char=="tab":
            self.char='\t'
        elif char=="space":
            self.char=' '
        else:
            self.char = char

def keytext_to_keyinfo_and_event (keytext):
    keyinfo = keysyms.common.make_KeyPress_from_keydescr (keytext)
    if len(keytext) == 3 and keytext[0] == '"' and keytext[2] == '"':
        event = Event (keytext[1])
    else:
        event = Event (keyinfo.tuple() [3])
    return keyinfo, event



#override runTests from from main in unittest to remove sys.exit call
class Tester(unittest.TestProgram):
    def runTests(self):
        if self.testRunner is None:
            self.testRunner = unittest.TextTestRunner(verbosity=self.verbosity)
        result = self.testRunner.run(self.test)
#       sys.exit(not result.wasSuccessful())



# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Michael Graz. <mgraz@plan10.com>
#       Copyright (C) 2006  Michael Graz. <mgraz@plan10.com>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import

import sys, unittest
sys.path.insert(0, '../..')
from pyreadline.modes.emacs import *
from pyreadline import keysyms
from pyreadline.lineeditor import lineobj

from pyreadline.test.common import *
from pyreadline.logger import log
import pyreadline.logger as logger
logger.sock_silent=True
logger.show_event=["debug"]

#----------------------------------------------------------------------


class EmacsModeTest (EmacsMode):
    tested_commands={}
    def __init__ (self):
        EmacsMode.__init__ (self, MockReadline())
        self.mock_console = MockConsole ()
        self.init_editing_mode (None)
        self.lst_completions = []
        self.completer = self.mock_completer
        self.completer_delims = ' '
        self.tabstop = 4
        self.mark_directories=False
        self.show_all_if_ambiguous=False
        
    def get_mock_console (self):
        return self.mock_console
    console = property (get_mock_console)

    def _set_line (self, text):
        self.l_buffer.set_line (text)

    def get_line (self):
        return self.l_buffer.get_line_text ()
    line = property (get_line)

    def get_line_cursor (self):
        return self.l_buffer.point
    line_cursor = property (get_line_cursor)

    def input (self, keytext):
        if keytext[0:1] == '"' and keytext[-1:] == '"':
            lst_key = ['"%s"' % c for c in keytext[1:-1]]
        else:
            lst_key = [keytext]
        for key in lst_key:
            keyinfo, event = keytext_to_keyinfo_and_event (key)
            dispatch_func = self.key_dispatch.get(keyinfo.tuple(),self.self_insert)
            self.tested_commands[dispatch_func.__name__]=dispatch_func
            log("keydisp: %s %s"%( key,dispatch_func.__name__))
            dispatch_func (event)
            self.previous_func=dispatch_func

    def accept_line (self, e):
        if EmacsMode.accept_line (self, e):
            # simulate return
            # self.add_history (self.line)
            self.l_buffer.reset_line ()

    def mock_completer (self, text, state):
        return self.lst_completions [state]

#----------------------------------------------------------------------

class TestsKeyinfo (unittest.TestCase):

    def test_keyinfo (self):
        keyinfo, event = keytext_to_keyinfo_and_event ('"d"')
        self.assertEqual ('d', event.char)
        keyinfo, event = keytext_to_keyinfo_and_event ('"D"')
        self.assertEqual ('D', event.char)
        keyinfo, event = keytext_to_keyinfo_and_event ('"$"')
        self.assertEqual ('$', event.char)
        keyinfo, event = keytext_to_keyinfo_and_event ('Escape')
        self.assertEqual ('\x1b', event.char)


class TestsMovement (unittest.TestCase):
    def test_cursor (self):
        r = EmacsModeTest ()
        self.assertEqual (r.line, '')
        r.input('"First Second Third"')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Control-a')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 0)
        r.input('Control-e')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Home')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 0)
        r.input('Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 1)
        r.input('Ctrl-f')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 2)
        r.input('Ctrl-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 5)
        r.input('Ctrl-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 12)
        r.input('Ctrl-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Ctrl-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Ctrl-Left')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 13)
        r.input('Ctrl-Left')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 6)
        r.input('Ctrl-Left')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 0)
        r.input('Ctrl-Left')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 0)


class TestsDelete (unittest.TestCase):
    def test_delete (self):
        r = EmacsModeTest ()
        self.assertEqual (r.line, '')
        r.input('"First Second Third"')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Delete')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Left')
        r.input('Left')
        r.input('Delete')
        self.assertEqual (r.line, 'First Second Thid')
        self.assertEqual (r.line_cursor, 16)
        r.input('Delete')
        self.assertEqual (r.line, 'First Second Thi')
        self.assertEqual (r.line_cursor, 16)
        r.input('Backspace')
        self.assertEqual (r.line, 'First Second Th')
        self.assertEqual (r.line_cursor, 15)
        r.input('Home')
        r.input('Right')
        r.input('Right')
        self.assertEqual (r.line, 'First Second Th')
        self.assertEqual (r.line_cursor, 2)
        r.input('Backspace')
        self.assertEqual (r.line, 'Frst Second Th')
        self.assertEqual (r.line_cursor, 1)
        r.input('Backspace')
        self.assertEqual (r.line, 'rst Second Th')
        self.assertEqual (r.line_cursor, 0)
        r.input('Backspace')
        self.assertEqual (r.line, 'rst Second Th')
        self.assertEqual (r.line_cursor, 0)
        r.input('Escape')
        self.assertEqual (r.line, '')
        self.assertEqual (r.line_cursor, 0)
        
    def test_delete_word (self):
        r = EmacsModeTest ()
        self.assertEqual (r.line, '')
        r.input('"First Second Third"')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        r.input('Control-Backspace')
        self.assertEqual (r.line, 'First Second ')
        self.assertEqual (r.line_cursor, 13)
        r.input('Backspace')
        r.input('Left')
        r.input('Left')
        self.assertEqual (r.line, 'First Second')
        self.assertEqual (r.line_cursor, 10)
        r.input('Control-Backspace')
        self.assertEqual (r.line, 'First nd')
        self.assertEqual (r.line_cursor, 6)
        r.input('Escape')
        self.assertEqual (r.line, '')
        self.assertEqual (r.line_cursor, 0)
        r.input('"First Second Third"')
        r.input('Home')
        r.input('Right')
        r.input('Right')
        r.input('Control-Delete')
        self.assertEqual (r.line, 'FiSecond Third')
        self.assertEqual (r.line_cursor, 2)
        r.input('Control-Delete')
        self.assertEqual (r.line, 'FiThird')
        self.assertEqual (r.line_cursor, 2)
        r.input('Control-Delete')
        self.assertEqual (r.line, 'Fi')
        self.assertEqual (r.line_cursor, 2)
        r.input('Control-Delete')
        self.assertEqual (r.line, 'Fi')
        self.assertEqual (r.line_cursor, 2)
        r.input('Escape')
        self.assertEqual (r.line, '')
        self.assertEqual (r.line_cursor, 0)



class TestsSelectionMovement (unittest.TestCase):
    def test_cursor (self):
        r = EmacsModeTest ()
        self.assertEqual (r.line, '')
        r.input('"First Second Third"')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 18)
        self.assertEqual (r.l_buffer.selection_mark, -1)
        r.input('Home')
        r.input('Shift-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 1)
        self.assertEqual (r.l_buffer.selection_mark, 0)
        r.input('Shift-Control-Right')
        self.assertEqual (r.line, 'First Second Third')
        self.assertEqual (r.line_cursor, 5)
        self.assertEqual (r.l_buffer.selection_mark, 0)
        r.input('"a"')
        self.assertEqual (r.line, 'a Second Third')
        self.assertEqual (r.line_cursor, 1)
        self.assertEqual (r.l_buffer.selection_mark, -1)
        r.input('Shift-End')
        self.assertEqual (r.line, 'a Second Third')
        self.assertEqual (r.line_cursor, 14)
        self.assertEqual (r.l_buffer.selection_mark, 1)
        r.input('Delete')
        self.assertEqual (r.line, 'a')
        self.assertEqual (r.line_cursor, 1)
        self.assertEqual (r.l_buffer.selection_mark, -1)



class TestsHistory (unittest.TestCase):
    def test_history_1 (self):
        r = EmacsModeTest ()
        r.add_history ('aa')
        r.add_history ('bbb')
        self.assertEqual (r.line, '')
        r.input ('Up')
        self.assertEqual (r.line, 'bbb')
        self.assertEqual (r.line_cursor, 3)
        r.input ('Up')
        self.assertEqual (r.line, 'aa')
        self.assertEqual (r.line_cursor, 2)
        r.input ('Up')
        self.assertEqual (r.line, 'aa')
        self.assertEqual (r.line_cursor, 2)
        r.input ('Down')
        self.assertEqual (r.line, 'bbb')
        self.assertEqual (r.line_cursor, 3)
        r.input ('Down')
        self.assertEqual (r.line, '')
        self.assertEqual (r.line_cursor, 0)

    def test_history_2 (self):
        r = EmacsModeTest ()
        r.add_history ('aaaa')
        r.add_history ('aaba')
        r.add_history ('aaca')
        r.add_history ('akca')
        r.add_history ('bbb')
        r.add_history ('ako')
        self.assert_line(r,'',0)
        r.input ('"a"')
        r.input ('Up')
        self.assert_line(r,'ako',1)
        r.input ('Up')
        self.assert_line(r,'akca',1)
        r.input ('Up')
        self.assert_line(r,'aaca',1)
        r.input ('Up')
        self.assert_line(r,'aaba',1)
        r.input ('Up')
        self.assert_line(r,'aaaa',1)
        r.input ('Right')
        self.assert_line(r,'aaaa',2)
        r.input ('Down')
        self.assert_line(r,'aaba',2)
        r.input ('Down')
        self.assert_line(r,'aaca',2)
        r.input ('Down')
        self.assert_line(r,'aaca',2)
        r.input ('Left')
        r.input ('Left')
        r.input ('Down')
        r.input ('Down')
        self.assert_line(r,'bbb',3)
        r.input ('Left')
        self.assert_line(r,'bbb',2)
        r.input ('Down')
        self.assert_line(r,'bbb',2)
        r.input ('Up')
        self.assert_line(r,'bbb',2)


    def test_history_3 (self):
        r = EmacsModeTest ()
        r.add_history ('aaaa')
        r.add_history ('aaba')
        r.add_history ('aaca')
        r.add_history ('akca')
        r.add_history ('bbb')
        r.add_history ('ako')
        self.assert_line(r,'',0)
        r.input ('')
        r.input ('Up')
        self.assert_line(r,'ako',3)
        r.input ('Down')
        self.assert_line(r,'',0)
        r.input ('Up')
        self.assert_line(r,'ako',3)

    def test_history_3 (self):
        r = EmacsModeTest ()
        r.add_history ('aaaa')
        r.add_history ('aaba')
        r.add_history ('aaca')
        r.add_history ('akca')
        r.add_history ('bbb')
        r.add_history ('ako')
        self.assert_line(r,'',0)
        r.input ('k')
        r.input ('Up')
        self.assert_line(r,'k',1)

    def test_complete (self):
        import rlcompleter
        logger.sock_silent = False

        log("-" * 50)
        r = EmacsModeTest()
        completerobj = rlcompleter.Completer()
        def _nop(val, word):
            return word
        completerobj._callable_postfix = _nop
        r.completer = completerobj.complete
        r._bind_key("tab", r.complete)
        r.input('"exi(ksdjksjd)"')
        r.input('Control-a')
        r.input('Right')
        r.input('Right')
        r.input('Right')
        r.input('Tab')
        self.assert_line(r, "exit(ksdjksjd)", 4)

        r.input('Escape')
        r.input('"exi"')
        r.input('Control-a')
        r.input('Right')
        r.input('Right')
        r.input('Right')
        r.input('Tab')
        self.assert_line(r, "exit", 4)

        
        
    def assert_line(self,r,line,cursor):
        self.assertEqual (r.line, line)
        self.assertEqual (r.line_cursor, cursor)
        
#----------------------------------------------------------------------
# utility functions

#----------------------------------------------------------------------

if __name__ == '__main__':
    Tester()
    tested=list(EmacsModeTest.tested_commands.keys())    
    tested.sort()
#    print(" Tested functions ".center(60,"-"))
#    print( "\n".join(tested))
#    print()
    
    all_funcs=dict([(x.__name__,x) for x in list(EmacsModeTest().key_dispatch.values())])
    all_funcs=list(all_funcs.keys())
    not_tested=[x for x in all_funcs if x not in tested]
    not_tested.sort()
    print(" Not tested functions ".center(60,"-"))
    print("\n".join(not_tested))



from __future__ import print_function, unicode_literals, absolute_import
__all__=["emacs", "notemacs", "vi"]
from . import emacs, notemacs, vi
editingmodes = [emacs.EmacsMode, notemacs.NotEmacsMode, vi.ViMode]

#add check to ensure all modes have unique mode names


# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import


class ReadlineError(Exception):
    pass

class GetSetError(ReadlineError):
    pass



from __future__ import print_function, unicode_literals, absolute_import
import sys, textwrap
from .py3k_compat import callable

rlmain = sys.modules["readline"]
rl = rlmain.rl

def get_doc(rl):
    methods = [(x, getattr(rl, x)) for x in dir(rl) if callable(getattr(rl, x))]
    return [ (x, m.__doc__ )for x, m in methods if m.__doc__]
    
    
def get_rest(rl):
    q = get_doc(rl)
    out = []
    for funcname, doc in q:
        out.append(funcname)
        out.append("\n".join(textwrap.wrap(doc, 80, initial_indent="   ")))
        out.append("")
    return out     


# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2003-2006 Gary Bishop.
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from __future__ import print_function, unicode_literals, absolute_import

'''Cursor control and color for the .NET console.
'''

#
# Ironpython requires a patch to work do:
#
# In file PythonCommandLine.cs patch line:     
#    class PythonCommandLine
#    {

# to:
#    public class PythonCommandLine
#    {
#
#
#
# primitive debug printing that won't interfere with the screen

import clr,sys
clr.AddReferenceToFileAndPath(sys.executable)
import IronPythonConsole

import sys
import re
import os

import System

from .event import Event
from pyreadline.logger import log

from pyreadline.keysyms import \
    make_keysym, make_keyinfo, make_KeyPress, make_KeyPress_from_keydescr
from pyreadline.console.ansi import AnsiState
color = System.ConsoleColor

ansicolor={"0;30": color.Black,
           "0;31": color.DarkRed,
           "0;32": color.DarkGreen,
           "0;33": color.DarkYellow,
           "0;34": color.DarkBlue,
           "0;35": color.DarkMagenta,
           "0;36": color.DarkCyan,
           "0;37": color.DarkGray,
           "1;30": color.Gray,
           "1;31": color.Red,
           "1;32": color.Green,
           "1;33": color.Yellow,
           "1;34": color.Blue,
           "1;35": color.Magenta,
           "1;36": color.Cyan,
           "1;37": color.White
          }

winattr = {"black" : 0,        "darkgray" : 0+8,
           "darkred" : 4,      "red" : 4+8,
           "darkgreen" : 2,    "green" : 2+8,
           "darkyellow" : 6,   "yellow" : 6+8,
           "darkblue" : 1,     "blue" : 1+8,
           "darkmagenta" : 5,  "magenta" : 5+8,
           "darkcyan" : 3,     "cyan" : 3+8,
           "gray" : 7,         "white" : 7+8}

class Console(object):
    '''Console driver for Windows.

    '''

    def __init__(self, newbuffer=0):
        '''Initialize the Console object.

        newbuffer=1 will allocate a new buffer so the old content will be restored
        on exit.
        '''
        self.serial = 0
        self.attr = System.Console.ForegroundColor
        self.saveattr = winattr[str(System.Console.ForegroundColor).lower()]
        self.savebg = System.Console.BackgroundColor
        log('initial attr=%s' % self.attr)

    def _get(self):
        top = System.Console.WindowTop
        log("WindowTop:%s"%top)
        return top

    def _set(self, value):
        top = System.Console.WindowTop
        log("Set WindowTop:old:%s,new:%s"%(top, value))

    WindowTop = property(_get, _set)
    del _get, _set

    def __del__(self):
        '''Cleanup the console when finished.'''
        # I don't think this ever gets called
        pass

    def pos(self, x=None, y=None):
        '''Move or query the window cursor.'''
        if x is not None:
            System.Console.CursorLeft=x
        else:
            x = System.Console.CursorLeft
        if y is not None:
            System.Console.CursorTop=y
        else:
            y = System.Console.CursorTop
        return x, y

    def home(self):
        '''Move to home.'''
        self.pos(0, 0)

# Map ANSI color escape sequences into Windows Console Attributes

    terminal_escape = re.compile('(\001?\033\\[[0-9;]*m\002?)')
    escape_parts = re.compile('\001?\033\\[([0-9;]*)m\002?')

    # This pattern should match all characters that change the cursor position differently
    # than a normal character.
    motion_char_re = re.compile('([\n\r\t\010\007])')

    def write_scrolling(self, text, attr=None):
        '''write text at current cursor position while watching for scrolling.

        If the window scrolls because you are at the bottom of the screen
        buffer, all positions that you are storing will be shifted by the
        scroll amount. For example, I remember the cursor position of the
        prompt so that I can redraw the line but if the window scrolls,
        the remembered position is off.

        This variant of write tries to keep track of the cursor position
        so that it will know when the screen buffer is scrolled. It
        returns the number of lines that the buffer scrolled.

        '''
        x, y = self.pos()
        w, h = self.size()
        scroll = 0 # the result

        # split the string into ordinary characters and funny characters
        chunks = self.motion_char_re.split(text)
        for chunk in chunks:
            n = self.write_color(chunk, attr)
            if len(chunk) == 1: # the funny characters will be alone
                if chunk[0] == '\n': # newline
                    x = 0
                    y += 1
                elif chunk[0] == '\r': # carriage return
                    x = 0
                elif chunk[0] == '\t': # tab
                    x = 8 * (int(x / 8) + 1)
                    if x > w: # newline
                        x -= w
                        y += 1
                elif chunk[0] == '\007': # bell
                    pass
                elif chunk[0] == '\010':
                    x -= 1
                    if x < 0:
                        y -= 1 # backed up 1 line
                else: # ordinary character
                    x += 1
                if x == w: # wrap
                    x = 0
                    y += 1
                if y == h: # scroll
                    scroll += 1
                    y = h - 1
            else: # chunk of ordinary characters
                x += n
                l = int(x / w) # lines we advanced
                x = x % w # new x value
                y += l
                if y >= h: # scroll
                    scroll += y - h + 1
                    y = h - 1
        return scroll

    trtable = {0 : color.Black,      4 : color.DarkRed,  2 : color.DarkGreen,
               6 : color.DarkYellow, 1 : color.DarkBlue, 5 : color.DarkMagenta,  
               3 : color.DarkCyan,   7 : color.Gray,     8 : color.DarkGray,   
               4+8 : color.Red,      2+8 : color.Green,  6+8 : color.Yellow,
               1+8 : color.Blue,     5+8 : color.Magenta,3+8 : color.Cyan,
               7+8 : color.White}

    def write_color(self, text, attr=None):
        '''write text at current cursor position and interpret color escapes.

        return the number of characters written.
        '''
        log('write_color("%s", %s)' % (text, attr))
        chunks = self.terminal_escape.split(text)
        log('chunks=%s' % repr(chunks))
        bg = self.savebg
        n = 0 # count the characters we actually write, omitting the escapes
        if attr is None:#use attribute from initial console
            attr = self.attr
        try:
            fg = self.trtable[(0x000f&attr)]
            bg = self.trtable[(0x00f0&attr)>>4]
        except TypeError:
            fg = attr
            
        for chunk in chunks:
            m = self.escape_parts.match(chunk)
            if m:
                log(m.group(1))
                attr = ansicolor.get(m.group(1), self.attr)
            n += len(chunk)
            System.Console.ForegroundColor = fg
            System.Console.BackgroundColor = bg
            System.Console.Write(chunk)
        return n

    def write_plain(self, text, attr=None):
        '''write text at current cursor position.'''
        log('write("%s", %s)' %(text, attr))
        if attr is None:
            attr = self.attr
        n = c_int(0)
        self.SetConsoleTextAttribute(self.hout, attr)
        self.WriteConsoleA(self.hout, text, len(text), byref(n), None)
        return len(text)
        
    if "EMACS" in os.environ:
        def write_color(self, text, attr=None):
            junk = c_int(0)
            self.WriteFile(self.hout, text, len(text), byref(junk), None)
            return len(text)
        write_plain = write_color

    # make this class look like a file object
    def write(self, text):
        log('write("%s")' % text)
        return self.write_color(text)

    #write = write_scrolling

    def isatty(self):
        return True

    def flush(self):
        pass

    def page(self, attr=None, fill=' '):
        '''Fill the entire screen.'''
        System.Console.Clear()

    def text(self, x, y, text, attr=None):
        '''Write text at the given position.'''
        self.pos(x, y)
        self.write_color(text, attr)

    def clear_to_end_of_window(self):
        oldtop = self.WindowTop
        lastline = self.WindowTop+System.Console.WindowHeight
        pos = self.pos()
        w, h = self.size()
        length = w - pos[0] + min((lastline - pos[1] - 1), 5) * w - 1
        self.write_color(length * " ")
        self.pos(*pos)
        self.WindowTop = oldtop
        
    def rectangle(self, rect, attr=None, fill=' '):
        '''Fill Rectangle.'''
        oldtop = self.WindowTop
        oldpos = self.pos()
        #raise NotImplementedError
        x0, y0, x1, y1 = rect
        if attr is None:
            attr = self.attr
        if fill:
            rowfill = fill[:1] * abs(x1 - x0)
        else:
            rowfill = ' ' * abs(x1 - x0)
        for y in range(y0, y1):
                System.Console.SetCursorPosition(x0, y)
                self.write_color(rowfill, attr)
        self.pos(*oldpos)

    def scroll(self, rect, dx, dy, attr=None, fill=' '):
        '''Scroll a rectangle.'''
        raise NotImplementedError

    def scroll_window(self, lines):
        '''Scroll the window by the indicated number of lines.'''
        top = self.WindowTop + lines
        if top < 0:
            top = 0
        if top + System.Console.WindowHeight > System.Console.BufferHeight:
            top = System.Console.BufferHeight
        self.WindowTop = top

    def getkeypress(self):
        '''Return next key press event from the queue, ignoring others.'''
        ck = System.ConsoleKey
        while 1:
            e = System.Console.ReadKey(True)
            if e.Key == System.ConsoleKey.PageDown: #PageDown
                self.scroll_window(12)
            elif e.Key == System.ConsoleKey.PageUp:#PageUp
                self.scroll_window(-12)
            elif str(e.KeyChar) == "\000":#Drop deadkeys
                log("Deadkey: %s"%e)
                return event(self, e)
            else:
                return event(self, e)

    def title(self, txt=None):
        '''Set/get title.'''
        if txt:
            System.Console.Title = txt
        else:
            return System.Console.Title

    def size(self, width=None, height=None):
        '''Set/get window size.'''
        sc = System.Console
        if width is not None and height is not None:
            sc.BufferWidth, sc.BufferHeight = width,height
        else:
            return sc.BufferWidth, sc.BufferHeight

        if width is not None and height is not None:
            sc.WindowWidth, sc.WindowHeight = width,height
        else:
            return sc.WindowWidth - 1, sc.WindowHeight - 1
    
    def cursor(self, visible=True, size=None):
        '''Set cursor on or off.'''
        System.Console.CursorVisible = visible

    def bell(self):
        System.Console.Beep()

    def next_serial(self):
        '''Get next event serial number.'''
        self.serial += 1
        return self.serial

class event(Event):
    '''Represent events from the console.'''
    def __init__(self, console, input):
        '''Initialize an event from the Windows input structure.'''
        self.type = '??'
        self.serial = console.next_serial()
        self.width = 0
        self.height = 0
        self.x = 0
        self.y = 0
        self.char = str(input.KeyChar)
        self.keycode = input.Key
        self.state = input.Modifiers
        log("%s,%s,%s"%(input.Modifiers, input.Key, input.KeyChar))
        self.type = "KeyRelease"
        self.keysym = make_keysym(self.keycode)
        self.keyinfo = make_KeyPress(self.char, self.state, self.keycode)

def make_event_from_keydescr(keydescr):
    def input():
        return 1
    input.KeyChar = "a"
    input.Key = System.ConsoleKey.A
    input.Modifiers = System.ConsoleModifiers.Shift
    input.next_serial = input
    e = event(input,input)
    del input.next_serial
    keyinfo = make_KeyPress_from_keydescr(keydescr)
    e.keyinfo = keyinfo
    return e

CTRL_C_EVENT=make_event_from_keydescr("Control-c")

def install_readline(hook):
    def hook_wrap():
        try:
            res = hook()
        except KeyboardInterrupt as x:   #this exception does not seem to be caught
            res = ""
        except EOFError:
            return None
        if res[-1:] == "\n":
            return res[:-1]
        else:
            return res
    class IronPythonWrapper(IronPythonConsole.IConsole):
        def ReadLine(self, autoIndentSize): 
            return hook_wrap()
        def Write(self, text, style):
            System.Console.Write(text)
        def WriteLine(self, text, style): 
            System.Console.WriteLine(text)
    IronPythonConsole.PythonCommandLine.MyConsole = IronPythonWrapper()



if __name__ == '__main__':
    import time, sys
    c = Console(0)
    sys.stdout = c
    sys.stderr = c
    c.page()
    c.pos(5, 10)
    c.write('hi there')
    c.title("Testing console")
#    c.bell()
    print()
    print("size", c.size())
    print('  some printed output')
    for i in range(10):
        e = c.getkeypress()
        print(e.Key, chr(e.KeyChar), ord(e.KeyChar), e.Modifiers)
    del c

    System.Console.Clear()
